package com.tzld.longarticle.recommend.server.service.recommend.reduce;

import com.tzld.longarticle.recommend.server.common.ThreadPoolFactory;
import com.tzld.longarticle.recommend.server.service.recommend.reduce.strategy.SafeReduceStrategy;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import com.tzld.longarticle.recommend.server.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.TimeUnit;

@Service
@Slf4j
public class ReduceService implements ApplicationContextAware {

    private final Map<String, ReduceStrategy> strategyMap = new HashMap<>();
    private ApplicationContext applicationContext;
    private final ExecutorService pool = ThreadPoolFactory.reducePool();



    @PostConstruct
    public void init() {
        Map<String, ReduceStrategy> type = applicationContext.getBeansOfType(ReduceStrategy.class);
        for (Map.Entry<String, ReduceStrategy> entry : type.entrySet()) {
            ReduceStrategy value = entry.getValue();
            strategyMap.put(value.getClass().getSimpleName(), value);
        }
    }

    public void reduce(ReduceParam param) {
        long start = System.currentTimeMillis();
        List<ReduceStrategy> strategies = getReduceStrategy();
        log.info("ReduceStrategy {}", JSONUtils.toJson(CommonCollectionUtils.toList(strategies,
                s -> s.getClass().getSimpleName())));
        CountDownLatch cdl = new CountDownLatch(strategies.size());
        for (final ReduceStrategy strategy : strategies) {
            pool.submit(() -> {
                try {
                    strategy.reduce(param);
                } catch (Exception e) {
                    log.error("reduceService error:{}", e.getMessage(), e);
                } finally {
                    cdl.countDown();
                }
                return new ArrayList<>();
            });
        }
        try {
            cdl.await(30, TimeUnit.SECONDS);
        } catch (InterruptedException e) {
            log.error("reduce error", e);
            return;
        }
        param.getContent().sort((o1, o2) -> -Double.compare(o1.getScore(), o2.getScore()));
        log.info("reduce耗时：{}", System.currentTimeMillis() - start);
    }

    private List<ReduceStrategy> getReduceStrategy() {
        List<ReduceStrategy> strategies = new ArrayList<>();
        strategies.add(strategyMap.get(SafeReduceStrategy.class.getSimpleName()));
        return strategies;
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        this.applicationContext = applicationContext;
    }


}
